﻿using Microsoft.VisualBasic;
using System;
using System.Web;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
public class PersistanceDataSupport
{

    #region " GetPersistanceValue "
    /// <summary>
    /// A session variable, or if exists, a querystring variable by the same name.  Data in the querystring with the same key will override anything in session with the same name.
    /// </summary>
    /// <typeparam name="T">The datatype expected</typeparam>
    /// <param name="PersistanceKey">The name to identify the variable.</param>
    /// <param name="UseSession">True if this value should exist in the session state.  If it wasn't already there at the time of this call, then it will be added to session state from the data in the querystring.</param>
    public static Nullable<T> GetPersistanceValue<T>(string PersistanceKey, bool UseSession) where T : struct
    {
        string sTemp = HttpContext.Current.Request.QueryString[PersistanceKey];
        T TestValue = default(T);

        if (sTemp != null && GenericTryCast<T>(sTemp, ref TestValue))
        {
            if (UseSession)
            {
                SetSessionValue<T>(PersistanceKey, TestValue);
            }

            return TestValue;
        }
        else if (UseSession && HttpContext.Current.Session[PersistanceKey] != null)
        {
            return (T)HttpContext.Current.Session[PersistanceKey];
        }
        else
        {
            return null;
        }
    }

    /// <summary>
    /// A session variable, or if exists, a querystring variable by the same name.  Data in the querystring with the same key will override anything in session with the same name.
    /// </summary>
    /// <param name="PersistanceKey">The name to identify the variable.</param>
    /// <param name="UseSession">True if this value should exist in the session state.  If it wasn't already there at the time of this call, then it will be added to session state from the data in the querystring.</param>
    public static string GetPersistanceValue(string PersistanceKey, bool UseSession)
    {
        string sTemp = HttpContext.Current.Request.QueryString[PersistanceKey];

        if (sTemp != null)
        {
            if (UseSession)
            {
                SetSessionValue(PersistanceKey, sTemp);
            }

            return sTemp;
        }
        else if (UseSession && HttpContext.Current.Session[PersistanceKey] != null)
        {
            return (string)HttpContext.Current.Session[PersistanceKey];
        }
        else
        {
            return null;
        }
    }
    #endregion

    #region " GenericTryCast "
    /// <summary>
    /// Attempt to cast a value to a specific type similiar to the "TryCast" method available on many structures.
    /// </summary>
    /// <typeparam name="T">The datatype to cast to.</typeparam>
    /// <param name="Value">The input value to cast from</param>
    /// <param name="Result">The result to store a the new value in if the cast is successful</param>
    public static bool GenericTryCast<T>(object Value, ref T Result)
    {
        try
        {
            Result = (T)Convert.ChangeType(Value, typeof(T));
        }
        catch
        {
            return false;
        }

        return true;
    }
    #endregion

    #region " SetSessionValue "
    /// <summary>
    /// Writes a value to session.
    /// </summary>
    /// <param name="PersistanceKey">The name of the session value.</param>
    /// <param name="Value">The value to save.</param>
    public static void SetSessionValue(string PersistanceKey, string Value)
    {
        HttpContext.Current.Session[PersistanceKey] = Value;
    }

    /// <summary>
    /// Writes a value to session.
    /// </summary>
    /// <typeparam name="T">The datatype of the session value.</typeparam>
    /// <param name="PersistanceKey">The name of the session value.</param>
    /// <param name="Value">The value to save.</param>
    public static void SetSessionValue<T>(string PersistanceKey, Nullable<T> Value) where T : struct
    {
        if (Value.HasValue)
        {
            HttpContext.Current.Session[PersistanceKey] = Value.Value;
        }
        else
        {
            HttpContext.Current.Session[PersistanceKey] = null;
        }
    }
    #endregion

}
