﻿#Region "CompilerDefaults"
Option Strict On
Option Explicit On
Option Infer On
Option Compare Binary
#End Region

Public Class PersistanceDataSupport

#Region " GetPersistanceValue "
    ''' <summary>
    ''' A session variable, or if exists, a querystring variable by the same name.  Data in the querystring with the same key will override anything in session with the same name.
    ''' </summary>
    ''' <typeparam name="T">The datatype expected</typeparam>
    ''' <param name="PersistanceKey">The name to identify the variable.</param>
    ''' <param name="UseSession">True if this value should exist in the session state.  If it wasn't already there at the time of this call, then it will be added to session state from the data in the querystring.</param>
    Public Overloads Shared Function GetPersistanceValue(Of T As Structure)(PersistanceKey As String, UseSession As Boolean) As Nullable(Of T)
        Dim sTemp = HttpContext.Current.Request.QueryString(PersistanceKey)
        Dim TestValue As T

        If sTemp IsNot Nothing AndAlso GenericTryCast(Of T)(sTemp, TestValue) Then
            If UseSession Then
                SetSessionValue(PersistanceKey, TestValue)
            End If

            Return TestValue
        ElseIf UseSession AndAlso HttpContext.Current.Session(PersistanceKey) IsNot Nothing Then
            Return DirectCast(HttpContext.Current.Session(PersistanceKey), T)
        Else
            Return Nothing
        End If
    End Function

    ''' <summary>
    ''' A session variable, or if exists, a querystring variable by the same name.  Data in the querystring with the same key will override anything in session with the same name.
    ''' </summary>
    ''' <param name="PersistanceKey">The name to identify the variable.</param>
    ''' <param name="UseSession">True if this value should exist in the session state.  If it wasn't already there at the time of this call, then it will be added to session state from the data in the querystring.</param>
    Public Overloads Shared Function GetPersistanceValue(PersistanceKey As String, UseSession As Boolean) As String
        Dim sTemp = HttpContext.Current.Request.QueryString(PersistanceKey)

        If sTemp IsNot Nothing Then
            If UseSession Then
                SetSessionValue(PersistanceKey, sTemp)
            End If

            Return sTemp
        ElseIf UseSession AndAlso HttpContext.Current.Session(PersistanceKey) IsNot Nothing Then
            Return DirectCast(HttpContext.Current.Session(PersistanceKey), String)
        Else
            Return Nothing
        End If
    End Function
#End Region

#Region " GenericTryCast "
    ''' <summary>
    ''' Attempt to cast a value to a specific type similiar to the "TryCast" method available on many structures.
    ''' </summary>
    ''' <typeparam name="T">The datatype to cast to.</typeparam>
    ''' <param name="Value">The input value to cast from</param>
    ''' <param name="Result">The result to store a the new value in if the cast is successful</param>
    Public Shared Function GenericTryCast(Of T)(Value As Object, ByRef Result As T) As Boolean
        Try
            Result = DirectCast(Convert.ChangeType(Value, GetType(T)), T)
        Catch
            Return False
        End Try

        Return True
    End Function
#End Region

#Region " SetSessionValue "
    ''' <summary>
    ''' Writes a value to session.
    ''' </summary>
    ''' <param name="PersistanceKey">The name of the session value.</param>
    ''' <param name="Value">The value to save.</param>
    Public Overloads Shared Sub SetSessionValue(PersistanceKey As String, Value As String)
        HttpContext.Current.Session(PersistanceKey) = Value
    End Sub

    ''' <summary>
    ''' Writes a value to session.
    ''' </summary>
    ''' <typeparam name="T">The datatype of the session value.</typeparam>
    ''' <param name="PersistanceKey">The name of the session value.</param>
    ''' <param name="Value">The value to save.</param>
    Public Overloads Shared Sub SetSessionValue(Of T As Structure)(PersistanceKey As String, Value As Nullable(Of T))
        If Value.HasValue Then
            HttpContext.Current.Session(PersistanceKey) = Value.Value
        Else
            HttpContext.Current.Session(PersistanceKey) = Nothing
        End If
    End Sub
#End Region

End Class